#!/usr/bin/perl
# mkdoc.pl

=pod

=head1 NAME

mkdoc.pl - Documentation creator.

=head1 SYNOPSIS

C<perl mkdoc.pl>

=head1 DESCRIPTION

This program generates the documentation for every perl script in
the current directory. The perl scripts are identified by the C<.pl>
extension of the filename.

The whole documentation is generated in the C<doc> subdirectory. If this
does not exist, it will be created.

A HTML index file to all created files will be created as C<index.html>,
so you can bookmark it in your browser and easily access the whole
documentation of the tools.

=head1 AUTHOR

Michael Winkelmann, michael@weltretter.de

=cut

use strict;
use warnings;

use Fatal qw(mkdir);
use Pod::Html;
use Pod::Text;
use Pod::LaTeX;
use File::Spec::Functions;
use DirHandle;


my $source_dir = ".";
my $doc_dir = catfile($source_dir, "doc");
my $general;
my $temp;
my @files;

(-d $source_dir) or die "$source_dir does not exist";

print "Creating documentation in the current directory\n";
(-d $doc_dir) or mkdir $doc_dir;

my $dh = new DirHandle($source_dir);
my $aktfile;
while($aktfile = $dh->read) {
    next unless $aktfile =~ m/^(.*)\.pl$/o;
    $general = $1;
    next if $general eq 'test';
    push @files, $general; 
    print "\nGenerating documentation for $aktfile\n";
    $temp = catfile($doc_dir, "$general.html");
    print "  Creating $temp\n";
    pod2html("--infile=$aktfile",
	     "--outfile=$temp");
    $temp = catfile($doc_dir, "$general.txt");
    print "  Creating $temp\n";
    my $parser = Pod::Text->new (sentence => 0, width => 72);
    $parser->parse_from_file($aktfile, $temp);
    $temp = catfile($doc_dir, "$general.tex");
    print "  Creating $temp\n";
    $parser = Pod::LaTeX->new();
    $parser->parse_from_file($aktfile, $temp);
}

print "\nGenerating documentation index\n";

$temp = catfile($doc_dir, 'index.html');
open F, ">$temp" or die "Could not open/create $temp for writing index.";

print F "<html>\n";
print F "<head><title>Voynich tools documentation index</title></head>\n";
print F "<body>\n";
print F "<h1>DOCUMENTATION INDEX</h1>\n";
print F "<table border=\"2\">\n";
print F "<tr><th>Tool name</th><th>Documentation</th></tr>\n";

foreach $general (@files)
{
    print F "<tr>\n";
    print F "<td>$general.pl</td>\n<td>";
    print F "<a href=\"$general.html\">HTML</a>&nbsp;";
    print F "<a href=\"$general.txt\">Plain&nbsp;text</a>&nbsp;";
    print F "<a href=\"$general.tex\">LaTeX</a></td>\n";
    print F "</tr>\n";
}

print F "</table>\n";
print F "</body>\n<html>\n";
close F;

print "\nDocumentation generation completed.\n";
print "You will find the documentation in the directory $doc_dir.\n";
