#!/usr/bin/perl
#########################################################################
# showeva.pl
# A script for displaying EVA transcriptions in the EVA Hand 1 font
# using the Tk toolkit.
#
# Version 0.04 (beta), 2004-09-16
# Copyright (c) 2004 Michael Winkelmann, michael@weltretter.de
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#########################################################################

=pod

=head1 NAME

showeva.pl - The EVA browser, version 0.4 beta

=head1 SYNOPSIS

C<perl showeva.pl [OPTIONS] [FILESPEC]>

=head1 DESCRIPTION

This is a browser for extracted Voynich MS transcriptions, designed 
to be used together with C<viat.pl>.

The extracted text can be viewed in a nice GUI interface.

=head1 OPTIONS

This program accepts mainly the GNU long option syntax. For normal
usage, there is no need for options at all, but sometimes you may
find them nice.

Everything you can set as an option, you can change at runtime
by using the GUI.

=over 4

=item C<--help> or C<-h> or C<-?>

Show the option overview.

=item C<--locators> or C<--nolocators>

Enable or disable the display of line locators. By default, the line
locators are enabled.

=item C<--evafont> or C<--noevafont>

Enable or disable the transcription display using the EVA Hand 1 font.
By defualt, the font is used.

=item C<--special> or C<--nospecial>

Enable or disable some special markup. At the moment, this is related
to the percent signs and the exclamation signs. By defualt, the special
markup is disabled.

=item C<--coments> or C<--nocomments>

Enable or disable the display of the meta-comments generated by the
C<viat.pl> tool with the C<-M> option. By defualt, the comments are
displayed.

=item C<--inline> or C<--noinline>

Enable or disable the display of the inline comments, written in
curly brackets. By default, the inline comments are displayed.

=back

=head1 FILE SPECIFICATION

If there is no specification, no file is read. You can select a file
in the C<File> menu.

If exactly one filename is given on the command line, the file will
be loaded and displayed in the GUI.

If the filename is a single dash, the file is read from the standard
input, so you can use the EVA browser in a pipe. 

=head1 BUGS

There are no known bugs at the moment. Some minor bugs from the 0.02
beta version are fixed, some new bugs may occur instead of them. 

The program was meant to be experimental, and the source is a mess. It
may be hard to add additional features.

=head1 TODO

It should be possible to generate C<viat.pl> output using a nice gui
and to save the generated text.

=head1 AUTHOR

showeva.pl was written by Michael Winkelmann, michael@weltretter.de

=cut


# Pragmas
use strict;
use warnings;

# Modules
use Getopt::Long qw(:config no_ignore_case bundling);
use Pod::Usage;
use File::Spec::Functions;

# GUI related modules
use Tk;
use Tk::Menu;
use Tk::Wm;
use Tk::Font;
use Tk::ROText;
use Tk::FileSelect;
use Tk::Balloon;


# VARIOUS GLOBAL HELPER VARIABLES
# -------------------------------

my $disp_locators = 1;
my $disp_evafont = 1;
my $disp_special = 0;
my $disp_comment = 1;
my $disp_inline = 1;
my $opt_help = 0;


# PARSE THE OPTIONS
# -----------------

GetOptions('locators!'    => \$disp_locators,
	   'evafont!'     => \$disp_evafont,
	   'specials!'    => \$disp_special,
	   'comments!'    => \$disp_comment,
	   'inlines!'     => \$disp_inline,
	   'help|h|?'     => \$opt_help)
    or die 'Illegal option found';

die "Too many arguments given" if($#ARGV > 0);

pod2usage(1) if ($opt_help);


# CREATE THE GUI ELEMENTS
# -----------------------

# Create the display window and the fonts needed.
my $top = new MainWindow;

# Create the tooltip generator
my $balloon = $top->Balloon(-state => 'balloon');
			    

# Create the fonts needed
my $font_hel = $top->Font(-family => 'Helvetica', 
			  -size => 8);
my $font_eva = $top->Font(-family => 'EVA Hand 1', 
			  -size => 14);
my $font_ehe = $top->Font(-family => 'Helvetica',
			  -size => 14);
my $font_spc = $top->Font(-family => 'Helvetica', 
			  -size => 11);
my $font_rem = $top->Font(-family => 'Courier',
			  -size => 8);
my $font_loc = $font_hel;

# Create the menus
my $menubar = $top->Menu;

my $mdef_file = 
    [ [ Button => 'Open...', 
	-underline => 0,
	-accelerator => 'Ctrl-O',
	-command => \&file_open_proc ],
      [ Separator => '' ],
      [ Button => 'Quit',
	-underline => 0,
	-accelerator => 'Ctrl-Q',
	-command => \&file_quit_proc ]
    ];

my $mdef_disp =
    [ [ Checkbutton => 'Line locators',
	-underline => 0,
	-accelerator => 'Ctrl-L',
	-command => \&disp_locators_proc,
	-variable => \$disp_locators ],
      [ Checkbutton => 'Meta comments',
	-underline => 5,
	-accelerator => 'Ctrl-K',
	-command => \&disp_comment_proc,
	-variable => \$disp_comment ],
      [ Checkbutton => 'Inline comments',
	-underline => 0,
	-accelerator => 'Ctrl-I',
	-command => \&disp_inline_proc,
	-variable => \$disp_inline ],
      [ Checkbutton => 'Special markups',
	-underline => 0,
	-accelerator => 'Ctrl-M',
	-command => \&disp_special_proc,
	-variable => \$disp_special ],
      [ Separator => '' ],
      [ Checkbutton => 'Use EVA font',
	-underline => 8,
	-accelerator => 'Ctrl-F',
	-command => \&disp_evafont_proc,
	-variable => \$disp_evafont ],
      ];

my $mdef_help = 
    [ [ Button => 'About...',
	-underline => 0,
	-command => \&help_about_proc ]
      ];

my $menu_file = $menubar->Menubutton(-text => 'File',
				     -underline => 0,
				     -menuitems => $mdef_file);

my $menu_disp = $menubar->Menubutton(-text => 'Display',
				     -underline => 0,
				     -menuitems => $mdef_disp);

my $menu_help = $menubar->Menubutton(-text => 'Help',
				     -underline => 0,
				     -menuitems => $mdef_help);

$top->configure(-menu => $menubar);

# Create the keyboard shortcuts
$top->bind('<Control-o>' => \&file_open_proc);
$top->bind('<Control-q>' => \&file_quit_proc);
$top->bind('<Control-f>' => sub { $disp_evafont ^= 1; disp_evafont_proc(); });
$top->bind('<Control-l>' => sub { $disp_locators ^= 1; disp_locators_proc();});
$top->bind('<Control-k>' => sub { $disp_comment ^= 1; disp_comment_proc(); });
$top->bind('<Control-m>' => sub { $disp_special ^= 1; disp_special_proc(); });
$top->bind('<Control-i>' => sub { $disp_inline ^= 1; disp_inline_proc(); });

# Create the toolbar of the main window
my $toolbar = $top->Frame(-borderwidth => 1,
			  -relief => 'raised');

# Create the Buttons in the toolbar
create_toolbar_button('Open a file for reading',
		      \&file_open_proc,
		      'open.xpm');
create_toolbar_spacer(4);
create_toolbar_checkbox('Display transcription text in EVA font', 
			\$disp_evafont, 
			\&disp_evafont_proc, 
			'font.xpm');
create_toolbar_checkbox('Display line locators',
			\$disp_locators,
			\&disp_locators_proc,
			'locator.xpm');
create_toolbar_checkbox('Display descriptive comments',
			\$disp_comment,
			\&disp_comment_proc,
			'comment.xpm');
create_toolbar_checkbox('Display inline comments',
			\$disp_inline,
			\&disp_inline_proc,
			'inline.xpm');
create_toolbar_checkbox('Display special spacing characters',
			\$disp_special,
			\&disp_special_proc,
			'special.xpm');
create_toolbar_button('Exit the EVA browser',
		      \&file_quit_proc,
		      'close.xpm', 
		      1);

$toolbar->pack(-fill => 'x',
	       -expand => 0);

# Create a scrolled text area for the EVA display
my $display = $top->Scrolled('ROText', 
			     -scrollbars => 'se');
$display->pack(-fill => 'both',
	       -expand => 1);

# Set the defaults for the EVA display
$display->configure(-font => $font_hel, 
		    -wrap => 'none',
		    -tabs => [qw/90 left/],
		    -width => 100, 
		    -background => '#F0F0F0',
		    -selectbackground => '#D0D0D0',
		    -selectborderwidth => 1,
		    -height => 27);

# Create the default tags for the voynich display
$display->tagConfigure('REM',                     # META-Remark
		       -foreground => '#802020',
		       -font => $font_rem); 
$display->tagConfigure('LOC',                     # Locator
		       -foreground => '#606060',
		       -font => $font_loc);
$display->tagConfigure('EVA',                     # EVA-Text
		       -foreground => '#000080',
		       -font => $font_eva);
$display->tagConfigure('SPC',                     # Space characters
		       -foreground => '#608060',
		       -font => $font_spc);
$display->tagConfigure('INR',                     # Inline comments
		       -foreground => '#802020',
		       -font => $font_spc);


# MAIN PROGRAM
# ------------

# Configure the tags initially, respecting options
disp_locators_proc();
disp_special_proc();
disp_comment_proc();
disp_inline_proc();
disp_evafont_proc();

# Set the focus to the main control of the application
$display->focus();
$display->markSet('insert', '1.0');

# Is there a filename or a dash on the command line - read the file
if($#ARGV == 0)
{
    my $filename = $ARGV[0];
    # Minus - read from standard input
    if($filename eq '-') {
	set_title('Reading from standard input...');
	while(<>) {
	    add_line($_);
	}
	set_title('Standard input display');
    }
    else {
	set_title("Reading file \"$filename\"...");
	while(<>) {
	    add_line($_);
	}
	set_title($filename);
    }
    $display->markSet('insert', '1.0');
}
else
{
    set_title();
}

# The Tk main loop
MainLoop;



# SUBROUTINES
# -----------

# Add line(s) to the Text area
sub add_line
{
    my ($line, $loc, $text, $item);

    foreach $line (@_) {
	chomp $line;
	# Ignore page descriptors
	next if $line =~ m/\{.*\$.*\}/o;
	# Handle special comments
	if($line =~ m/^\#-\s*(.+)$/o) {
	    $text = $1;
	    $display->insert('end',
			     "$text\n",
			     'REM');
	    next;
	}
	# Do not handle normal comments
	next if $line =~ m/^\s*\#/o;
	# Do nothing on empty lines
	next if $line =~ m/^\s*$/o;
	# Extract the locator and the text
	if($line =~ m/^\<([^\>]+)\>\s*(\S.*)$/o) {
	    $loc = $1;
	    $line = $2;
	    # Do nothing on lines without text
	    next if $line =~ m/^\s*$/o;
	    $display->insert('end',
			     "<$loc>\t",
			     'LOC');
	}
	else {
	    # No locator, strip the spaces from the line
	    $line =~ s/^\s*(\S.*)\s*$/$1/o;
	    $display->insert('end',
			     "\t",
			     'LOC');
	}
	# Create the standard ligatures - are they always correct?
	$line =~ s/cth/cTh/go;
	$line =~ s/cph/cPh/go;
	$line =~ s/cfh/cFh/go;
	$line =~ s/ckh/cKh/go;
	$line =~ s/ith/ITh/go;
	$line =~ s/iph/IPh/go;
	$line =~ s/ifh/IFh/go;
	$line =~ s/ikh/IKh/go;
	$line =~ s/sh/Sh/go;
	# Use normal spaces as word separators
	$line =~ s/\./ /go;
	# Add spaces before the "inline comment"
	$line =~ s/(\{[^\&])/ $1/go;
	# Split the line by the special EVA transcription characters.
	# The tokens are separated by semicolons.
	# $line =~ s/\=/\= /go;
	$line =~ s/(.\{\&[^\}]+\}|[\!\%\-\=]+)/;$1;/go;
	$line =~ s/(\{[^\}\&]*\})/;$1;/go;
	# Remove consecutive semicolons.
	$line =~ s/;{2,}/;/go;
	# Process the tokens
	foreach $item (split /\;/, $line) {
	    # Strip spaces from the tokens
	    $item =~ s/^\s*(\S.*\S)\s*$/$1/o;
	    next if($item =~ m/^\s*$/o);
	    # Special spacing characters
	    if($item =~ m/^[\!\%\-\=]/o) {
		$display->insert('end',
				 $item,
				 'SPC');
		if($item =~ m/\-/o) {
		    $display->insert('end',
				     ' ',
				     'EVA');
		}
	    }
	    # Special ligatures
	    elsif($item =~ m/^.\{\&(\D+)\}/o) {
		my $temp = $1;
		$display->insert('end',
				 $temp,
				 'EVA');
	    }
	    # Weirdos
	    elsif($item =~ m/^.\{\&(\d+)\}/o) {
		$display->insert('end', 
				 pack("c", $1),
				 'EVA');
	    }
	    # Inline comments
	    elsif($item =~ m/(\{[^\}]+\})/o) {
		my $temp = $1;
		$display->insert('end',
				 "$temp",
				 "INR");
	    }
	    # Normal EVA text
	    else {
		$display->insert('end',
				 $item,
				 'EVA');
	    }
	}
	# Append a newline and toggle the background color
	$display->insert('end', 
			 "\n",
			 "EVA");
	$top->update;
    }
}

# Set the program's window title
sub set_title
{
    my $files;
    my $title = "EVA browser:";
    if($#_ < 0) {
	$files = 'No file';
    }
    else {
	$files = join(' ', @_);
    }
    $top->title("$title $files");
}

# Exit the program
sub file_quit_proc
{
    Tk::exit(0);
}

# Request a file for opening and open it
sub file_open_proc
{
    my $types = [ [ 'All files',               '*' ],
		  [ 'Text files',              '.txt' ],
		  [ 'EVA transcription files', '.evt' ] ];
    my $filename = $top->getOpenFile(-title => 'Select an EVA file to open',
				     -filetypes => $types);
    if($filename) {
	if(open F, "<$filename") {
	    set_title("Reading file \"$filename\"...");
	    $display->delete('1.0', 'end');
	    while(<F>) {
		add_line($_);
	    }
	    close F;
	    set_title($filename);
	    $display->markSet('insert', '1.0');
	}
	else {
	    $top->messageBox(-icon => 'error',
			     -type => 'OK',
			     -title => 'Error reading file',
			     -message => "Could not open \"$filename\".");
	}
    }
}

# Toggle locator display
sub disp_locators_proc
{
    $display->tagConfigure('LOC', 
			   -state => $disp_locators ? 'normal' : 'hidden');
}

# Toggle meta comment display
sub disp_comment_proc
{
    $display->tagConfigure('REM',
			   -state => $disp_comment ? 'normal' : 'hidden');
}

# Toggle special markup display
sub disp_special_proc
{
    $display->tagConfigure('SPC',
			   -state => $disp_special ? 'normal' : 'hidden');
}

# Toggle inline comment display
sub disp_inline_proc
{
    $display->tagConfigure('INR',
			   -state => $disp_inline ? 'normal' : 'hidden');
}

# Toggle EVA font usage
sub disp_evafont_proc
{
    $display->tagConfigure('EVA',
			   -font => $disp_evafont ? $font_eva : $font_ehe);
}

# Show the about dialog
sub help_about_proc
{
    my $msg = 
	qq/showeva.pl, Version 0.4 beta\nCopyright (c) Michael Winkelmann/;
    $display->messageBox(-icon => 'info',
			 -title => 'About...',
			 -message => $msg,
			 -type => 'OK');
}


# Helper functions for the toolbar buttons
sub create_toolbar_checkbox
{
    my ($text, $vref, $sref, $xpmname) = @_;

    my $w = $toolbar->Checkbutton(-variable => $vref,
				  -command => $sref,
				  -indicatoron => 0);
    my $xpmpath = catfile('pic', $xpmname);
    my $xpm = $w->Pixmap(-file => $xpmpath);
    $w->configure(-image => $xpm);
    $w->pack(-side => 'left',
	     -padx => 2,
	     -pady => 2);
    $balloon->attach($w,
		     -msg => $text);
    return $w;
}

sub create_toolbar_button
{
    my ($text, $sref, $xpmname, $flag) = @_;

    my $w = $toolbar->Button(-command => $sref);
    my $xpmpath = catfile('pic', $xpmname);
    my $xpm = $w->Pixmap(-file => $xpmpath);
    $w->configure(-image => $xpm);
    $w->pack(-side => $flag ? 'right' : 'left',
	     -padx => 2,
	     -pady => 2);
    $balloon->attach($w,
		     -msg => $text);
    return $w;
}

sub create_toolbar_spacer
{
    my ($width) = @_;

    my $c = $toolbar->Frame(-width => $width);
    $c->pack(-side => 'left',
	     -padx => 2,
	     -pady => 2);
    return $c;
}
