#!/usr/bin/perl -w
#########################################################################
# vfind.pl
# Indexed search in the voynich interlinear transcription archive.
#
# Version 0.02 (beta), 2004-08-16
# Copyright (c) 2004 Michael Winkelmann, michael@weltretter.de
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#########################################################################

=pod

=head1 NAME

vfind.pl - Find occurences of a word in the voynich manuscript.

=head1 SYNOPSIS

C<perl vfind.pl [options] [wordlist]>

If you do not give a wordlist on the command line, the program runs in
interactive mode.

=head1 DESCRIPTION

The vfind.pl tool finds occurences of a word in the voynich manuscript. 
It uses the index generated by the vindex tool, so you have to generate
the index first.

=head1 OPTIONS

=over 4

=item C<-t transcriber-codes> or C<--transcriber=transcriber-codes>

Select only the pages with the given transcriber codes.

=item C<--interactive>

Switch to interactive mode after processing the words on the command
line.

=back

=head1 BUGS

No known bugs, but something you might find strange.

If the vindex.pl tool was invoked without options, it does not generate
index entries for words with asterisk characters. So only full readable
words are indexed.

=head1 AUTHOR

vfind.pl was written by Michael Winkelmann, michael@weltretter.de

=cut

use warnings;
use strict;
use Getopt::Long qw(:config bundling no_ignore_case);

use vsetting;


my %index;
my ($opt_t, $opt_i);



# Read the index file in a local hash
sub read_index_file
{
    my ($filename) = @_;
    my ($l, $word);

    (-f $filename) or die "$filename does not exist, please run vindex.pl";

    open IF, "<$filename" or die "Failed to open $filename";
    while($l = <IF>) {
	chomp $l;
	my @tmp_array = split / /, $l;
	$word = shift @tmp_array;
	@tmp_array = grep(m/\;[$opt_t]$/o, @tmp_array) if $opt_t;
	$index{$word} = \@tmp_array;
    }
    close IF or die "Could not close $filename";
}

# Print the occurences of a word
sub print_word_positions
{
    my ($word) = @_;
    my $i = 0;
    my $w;

    if(exists($index{$word})) {
	my @o = @{$index{$word}};
	printf "%s - %d occurences\n", $word, $#o + 1;
	foreach $w (@o) {
	    printf "%-16s ", $w;
	    if(++$i == 4) {
		print "\n";
		$i = 0;
	    }
	}
	print "\n" if($i != 0)
    }
    else {
	printf "%s does not exist\n", $word;
    }
    print "\n";
}

# Interactive mode
sub interactive_mode
{
    my $word;
    my $done = 0;

    print "vfind.pl interactive mode\n";
    print "Enter words to search or empty string to exit\n\n";
    while(!$done) {
	$word = <>;
	chomp $word;
	$word =~ s/^\s(.*)\s$/$1/go;
	if($word eq "") {
	    $done = 1;
	}
	else {
	    print_word_positions($word);
	}
    }
	
}


# MAIN PROGRAM
my $aktword;

GetOptions("transscriber|t=s"         => \$opt_t,
	   "interactive!"             => \$opt_i);

read_index_file($vsetting::indexfile);
if($#ARGV >= 0) {
    foreach $aktword (@ARGV) {
	print_word_positions($aktword);
    }
    interactive_mode() if $opt_i;
}
else {
    interactive_mode();
}

