#!/usr/bin/perl 
#########################################################################
# vhtml.pl
# A simple tool for formatting to HTML in a pipe.
#
# Version 0.2 (beta), 2004-09-08
# Copyright (c) 2004 Michael Winkelmann, michael@weltretter.de
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#########################################################################

=pod

=head1 NAME

vhtml.pl - The Voynich HTML formatter

=head1 SYNOPSIS

C<perl vhtml.pl [OPTIONS] [files]>

vhtml.pl is intended to be used in a pipe. It is ideal for collaboration
with viat.pl. But you can give a file on the command line. The output is
always written to the standard output, so you have to redirect or pipe
it to somewhere.

=head1 DESCRIPTION

For the case of analysis, the plain text representation of a Voynich
manuscript transcription fits perfect.

But if you want to look at the text or you want to compare the text to
some photos of the manuscript, you may find it helpful to see a nice
formatted HTML version using the EVA Hand 1 font.

And probably you do not want to see the comments (written in curly
brackets) formatted in the EVA font - it is not readable...

So I wrote this little script, which creates a nice formatted output.

The manuscript text is formatted in EVA Hand font and black color. 
It is designed to make reading easier by alternating background
colors in the lines.

Comments and line locators are formatted in a smaller font size and
grey color, so they should not disturb you, but you can read them.

Some special spacing characters, the C<!> and the C<%> are formatted
in a smaller font size and green color, so you can see their existance.

If the lowercased, basic EVA is used, some standard ligatures are generated
by uppercasing letters.

=head1 OPTIONS

=over 4

=item C<-t title> or C<--title=title>

Set the HTML document title.

There is no check for a legal title. You can use tags, illegal characters
or stranger things if you want to get unpredictable results.

If you do not set any title, the text "HTML formatted Voynich output" is
used.

=item C<b color-spec> or C<background=color-spec>

Set the background color of the resulting HTML document.

=item C<f color-spec> or C<foreground=color-spec>

Set the text color of the resulting HTML document.

The text color is used for EVA output, for the title and for
meta informations.

=item C<c color-spec> or C<comment=color-spec>

Set the comment color of the resulting HTML document.

The comment color is used for inlined comments and for the
line locators.

=item C<s color-spec> or C<space=color-spec>

Set the color for special space characters.

=item C<e color-spec> or C<evenrow=color-spec>

Set the background color for even numbered rows of the EVA output.

=item C<o color-spec> or C<oddrow=color-spec>

Set the background color for odd numbered rows of the EVA output.

=back

=head1 EXAMPLES

Create a HTML version of the Voynich manuscript transcription by
T. Takahashi with the viat.pl tool and the vhtml.pl tool.

C<perl viat.h -t H -w -M | perl vhtml.pl E<gt> takahasi.html>

=head1 BUGS

There is B<no error checking> at all.

The generated HTML could be nicer. At the moment, the whole output
is formatted in a huge HTML table, which the browser can not render
efficiently. I tried to use DIVs and positioning via CSS, but there are
serious bugs in most older browsers (as IE 5.0), so I decided to
format it in a table instead of forcing people to convert to Mozilla or
update their software.

The formatting is done via CSS. There is no FONT tag in the generated
HTML, because it is ugly. If your browser does not support CSS, try
Mozilla, Opera, Amaya or a Netscape version greater than 4.0. If you
use the old Netscape Navigator, expect ugly bugs and crashes - it is 
the most disgusting bugware I had ever seen (and I had seen a lot in
twenty years of computing experience). My Netscape 4.7 (under Linux 2.2
and XFree86 4) often crashes the whole X-Server, which is an unique
behaviour on my remarkable stable system.

If you do not have the EVA Hand 1 font, you see normal ASCII
characters. This is not a bug in my script, but it may cause
confusion.

The source is a mess. Adding more elaborated features may be
difficult - better rewrite it from scratch.

=head1 AUTHOR

Michael Winkelmann, michael@weltretter.de

=cut
;

use warnings;
use strict;
use Getopt::Long qw(:config bundling no_ignore_case);

# Parse the command line options
my ($opt_t, $opt_bg, $opt_fg, $opt_c, $opt_s, $opt_e, $opt_o);

GetOptions(
"title|t=s"         => \$opt_t,
"background|b=s"    => \$opt_bg,
"foreground|f=s"    => \$opt_fg,
"comment|c=s"       => \$opt_c,
"space|s=s"         => \$opt_s,
"evenrow|e=s"       => \$opt_e,
"oddrow|o=s"        => \$opt_o
);
$opt_t  = 'HTML formatted Voynich output' unless $opt_t;
$opt_bg = 'white'   unless $opt_bg;
$opt_fg = 'black'   unless $opt_fg;
$opt_c  = 'gray'    unless $opt_c;
$opt_s  = 'green'   unless $opt_s;
$opt_e  = '#f0f0f0' unless $opt_e;
$opt_o  = '#e0e0e0' unless $opt_o;

# Print HTML header with embedded CSS declarations
print <<EOF
<html>
<head>
<title>$opt_t</title>
<meta name="generator" content="vhtml.pl v0.2 beta">
<style type="text/css">
body 
{ 
    font-family:sans-serif; 
    color:$opt_fg; 
    background-color:$opt_bg; 
}
.eva 
{ 
    font-family:eva hand 1; 
    white-space:nowrap;
}
.cmt
{
    font-family:sans-serif;
    color:$opt_c;
    font-size:80%;
}
.spc
{
    font-family:sans-serif;
    color:$opt_s;
    font-size:80%;
}
.loc 
{ 
    font-family:monospace;
    color:$opt_c; 
    vertical-align:bottom;
    font-size:80%;
}
.meta 
{ 
    font-size:80%; 
    font-family:monospace;
    border:1px solid $opt_c;
    padding:8px;
}
.b0 
{
    background-color:$opt_e;
}
.b1
{
    background-color:$opt_o;
}
</style>
</head>
<body>
EOF
;

# Do processing - it is messy, don't read it, just use it...
my $tab = 0;
my $c = 0;
my ($l, $tmp, $txt, $loc);
print "<h3>$opt_t</h3>\n";
print "<table cellspacing=\"0\" cellpadding=\"2\" border=\"0\">\n";
while ($l = <>)
{
    chomp $l;

    # Handle meta comments from viat.pl
    if($l =~ m/^\#\-\s*(.*)$/o)
    {
        $tmp = $1;
        unless($tab)
	{
            $tab = 1;
	    print "<tr>\n";
	    print "<td class=\"meta\" colspan=\"2\">\n";
	}
	$tmp =~ s/ /&nbsp;/go;
        print "$tmp<br>\n";
        next;
    }

    # Close meta comment section, if open
    if($tab)
    {
        print "</td>\n";
        print "</tr>\n";
        $tab = $c = 0;
    }

    # Ignore empty, comment-line or "parseable information" lines
    next if $l =~ m/^\#/o or $l =~ m/\{\$/o or $l =~ m/^\s*$/o;

    # Create table rows with alternating background colors
    print "<tr class=\"b$c\">\n";
    $c ^= 1;

    # Handle the locator
    if($l =~ m/\<([^\>]+)\>\s*(.*)$/o)
    {
	# There is a locator - print it.
        $loc = $1;
        $txt = $2;
        print "<td class=\"loc\">&lt;$loc&gt;</td>\n";
    }
    else
    {
	# There is no locator - print an empty one for table layout
	print "<td class=\"loc\">&nbsp;</td>\n";
        $txt = $l;
    }

    # Remove trailing and leading spaces from the text
    $txt =~ s/^\s+//go;
    $txt =~ s/\s+$//go;

    # Replace consecutive spaces by a single space
    $txt =~ s/\s{2,}/ /go;

    # Do the text processing and output
    if($txt)
    {
	# Remove line terminators
	$txt =~ s/[-=]//go;

	# Replace weirdos by HTML entities
	$txt =~ s/.\{\&(\d+)\}/\&\#$1\;/go;
	$txt =~ s/.\{\&(\D{1,3})\}/$1/go;

	# Do some standard ligatures - is that always correct?
	$txt =~ s/cth/cTh/go;
	$txt =~ s/cph/cPh/go;
	$txt =~ s/cfh/cFh/go;
	$txt =~ s/ckh/cKh/go;
	$txt =~ s/ith/ITh/go;
	$txt =~ s/iph/IPh/go;
	$txt =~ s/ifh/IFh/go;
	$txt =~ s/ikh/IKh/go;
	$txt =~ s/sh/Sh/go;

	# Mark comments and special spacing characters
	$txt =~ s/\{([^\}]+)\}/&nbsp;<span class=\"cmt\">$1\<\/span>&nbsp;/go;
	$txt =~ s/([!%]+)/<span class=\"spc\">$1<\/span>/go;

	# Convert all remaining strange characters to HTML entites
	$txt =~ s/([^a-z0-9*&;.,\#<>\"=\/ ])/'&#'.ord($1).';'/goei;

	# Print the text
	print "<td class=\"eva\">$txt</td>\n";
    }
    else
    {
	# Print an empty table cell for the empty text
	print "<td class=\"eva\">&nbsp;</td>\n";
    }
    print "</tr>\n";
}
print "</table>\n";

# Print HTML footer
my $lt = localtime();
print <<EOF
<hr>
This file was created by the program vhtml.pl<br>
Creation time: $lt<br>
</body>
</html>
EOF
;
