#!/usr/bin/perl -w
#########################################################################
# viat.pl
# A simple tool for extracting text from the interlinear Voynich
# manuscript transcriptions in EVA, edited by stolfi.
#
# Version 0.03 (beta), 2004-09-12
# Copyright (c) 2004 Michael Winkelmann, michael@weltretter.de
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#########################################################################

# Some lines of documentation
=pod

=head1 NAME

viat.pl - The Voynich interlinear transcription archive tool

=head1 SYNOPSIS

C<perl viat.pl [OPTIONS]>

=head1 DESCRIPTION

This program is an extraction tool for the Voynich interlinear
transcription archive. It allows you to select interesting parts
from that file for your analysis.

The main purpose of the program is B<extraction>, not formatting.
So do not expect it to do fancy formatting. If you want to get a
nice formatted HTML document, pipe the output into the vhtml.pl
tool.

The program is intended to be used in a pipe, which is the most
natural way of data processing in unixoid systems. So the program
output is written to the standard output. 

=head1 OPTIONS

There are a plenty lot of options. If you want to see some easy to
use examples, take a look at L<"EXAMPLES">.

B<Please note> that all options and parameters are B<case-sensitive>, 
the C<-t> option is different from the C<-T> option, and specifing a
transcriber C<-T c> will silently select B<nothing> instead of the
Currier transcription.

You can give the options in two styles. The short, one-letter options
preceeded by a single dash in the Un*xoid style, and the long options 
preceeded by a double dash in the GNU style. Both styles are equivalent.

If you use the long options, you can abbreviate them as long they are
unique. 

=head2 Getting information

=over 4

=item C<-v> or C<--version>

Display the version of the program.

=item C<-I> or C<--show_illustration_codes>

Show all codes for illustration types and exit.

=item C<-T> or C<--show_transcriber_codes>

Show all code for transcribers and exit.

=back

=head2 General options

=over 4

=item C<-h> or C<-?> or C<--help>

Show an option overview as a short help.

=item C<-F path> or C<--file=path> or C<transcription=path>

Specify the path of the interlinear transcription file to use. By default,
the program looks for a file named 'text16e6.evt' in the current
directory.

=back

=head2 Output options

If you do not specify any output options, the extracted text is given
to the output in raw format, without any kind of processing. 

In most non-trivial cases, you want to do some processing.

=over 4

=item C<-c> or C<--comment_keep>

Keep all comment lines in the output. By default, they are suppressed.

=item C<-s> or C<--space_convert>

Convert all spacing (interpunctation, line, percent) characters to
real spaces.

=item C<-w> or C<--weirdo_convert>

Replace the weirdos by EVA codes whereever possible. Not every 
transcription has the correct weirdo notation.

=item C<-C> or C<--comment_suppress>

Remove all comments from the output text. Every text in curly brackets
which is not eventually replaced as an explicit weirdo is handled as
a comment. 

This feature may ease the job of processing the transcription.

=item C<-L> or C<--locator_suppress>

Do not output the line locator in front of a line of text.

=item C<-M> or C<--meta_comments>

Output some meta information for every page of the manuscript. Every
line of the meta information starts with the sequence #-, to make the
job of parsing easy. By convention, the # is handled as a comment line
marker, and so the meta information is extracted as a comment.

=item C<-S> or C<--dubious_suppress>

Remove all dubious word breaks in the output. If two Voynich "words" are 
separated only by a dubious space, they will be written to the output as
one word.

=back


=head2 Text selection options

If you do not specify any selection options, you will get the
whole text from the archive, only the comments are removed. This 
could be done much easier, so you probably want to B<select> some
text for the extraction.

The selection options gives you a way to extract folios, transcriptions,
Currier languages and hands.

=over 4

=item C<-f folio-range> or C<--folios=folio-range>

Show only the text from the given folio range. The range specification
can be given in the following ways (the letters n and m represents the
folio numbers):

=over 4

=item C<n>

If you specify a single folio number, only this folio will be
printed to the output.

=item C<n:m>

You can specify a range by using a colon between the folio
numbers. If your first page number is greater than your second
one, this type will be corrected silently.

=item C<:n>

If your specification starts with a colon, all folios from the first 
up to the given folio number will be printed.

=item C<n:>

If your specification ends with a colon, all folios beginning from the given
folio up to the last one will be printed.

=back

=item C<-i illustration-codes> or C<--illustration=illustration-codes>

Select only the pages with the given illustrations.

If you specify an unknown illustration code, this is not an error. It
selects silenty nothing.

=item C<-l language-code> or C<--language=language-code>

Select only the pages with the given Currier language code. The language
code is A or B.

There is not a Currier language associated to every page. If you
use this option, all pages without a known language will be hidden.

If you specify an unknown language code, it selects silently nothing.

=item C<-m hand-code> or C<--hand=hand-code>

Select only the pages with the given Currier hand code. The hand codes
are 1, 2, 3, 4, 5, A or B. You can combine more than one hand code.

There is not a Currier hand associated to every page. If you use
this option, all pages without a known hand will be hidden.

=item C<-t transcriber-codes> or C<transcriber=transcriber-codes>

Select only the text from the given transcriber. You can specify more
than one transcriber. The transcriber are given by single characters
with the following meaning:

=over 4

=item C

Currier

=item D

Currier, second choice

=item F

First study group

=item G

First study group, second choice

=item H

Takeshi Takahasi

=item I

Jim Reed, second choice

=item J

Jim Reed

=item K

Karl Kluge

=item L

Don Latham

=item M

Don Latham, second choice

=item N

Gabriel Landini

=item P

Father Th. Petersen

=item Q

Karl Kluge, second choice

=item R

Mike Roe

=item T

John Tiltman

=item U

Jorge Stolfi

=item V

John Grove

=item X

Denis V. Mardle

=item Z

Rene Zandbergen

=back

It is not an error, if you specify an unknown transcriber code. It just
does not select anything. If all given transcriber codes are unknown, 
you get an empty result.

By default, all transcribers are included in the result.

=back

=head1 EXAMPLES

Most examples are given in the short option syntax, which is fast to
type and hard to read. While doing text analysis on the Voynich 
manuscript and trying different approaches, this is most probably
the way you will type options on the prompt. 

But if you write some more complex scripts, which you want to share
with others, please use the much better readable long options. It will
help you and other people to understand what happens.

Extract only the Currier transcription.

C<viat.pl -t C>

Extract only the Currier transcription, but do not include the
line locators into the extraction. Calling the program this way, you
get the plain transcription text.

C<viat.pl -t C -L>

Extract only the Currier transcription for all pages with the
Currier language A.

C<viat.pl -t C -l A>

Extract only the Currier transcription, suppress the line locator,
replace all spacing characters by real spaces and replace all known
weirdos by the EVA character codes.

C<viat.pl -t C -L -s -w>

Do the same thing, but use the long option syntax.

C<viat.pl --transcriber=C --locator_suppress --space_convert>

Select all pages with currier language B from the T. Takahashi 
transcription.

C<viat.pl -t H -l B>

Do the same thing, but use the long option syntax.

C<viat.pl --transcriber=H --language=B> 

=head1 FILES

=over 4

=item text16e6.evt

The file containing the interlinear transcription. This file is expected to
be in your current directory by default, but you can specify any path and 
any file name using the -F option.

=back

=head1 BUGS

At the moment, there are no B<known> bugs. But there is a strange
behaviour, which does not affect the functionality of the program:

If you choose to include meta information comments in the extracted
file, you will get the information at the begin of every page, even
if there is no selected Voynich text in the page. It is not a bug,
because it only creates B<comments>, not B<contents>. But it is strange
and should be done better some day.

There is B<no good error checking> in the program. If, for example, the
parameter for an option is expected to be numeric, but you give it
some non-numeric value, the program silenty malfunctions. It should
not be a real problem, but it can cause confusion, especially if the
program is used in a pipe and the error gets not visible immedieatly.

=head1 TODO

I wrote this program as a personal replacement for the C<VTT> tool, which
is hard to read, hard to bugfix or change and hard to compile. There was
no thought on giving it away, while I was writing it. 

But in the mailing list I found that other people experiences problems
with the C<VTT> program too. So I wrote some documentation and prepared
the program for publishing. 

I expect that other users will miss some features that I do not need. Feel
free to contact me via mail, if you want an additional feature - I will
implement it, if the implementation is easy. But the feature should be a 
B<text extraction feature>, not a formatting or analysis feature.

Please keep in mind that I am not a native speaker of the English language
and do not expect me to understand every nuance of that language. I hope,
that my documentation is understandable to everyone who want to use
the program. If my english is clumsy or wrong, feel free to correct me.

=head1 AUTHOR

viat.pl was written by Michael Winkelmann, michael@weltretter.de

=cut

    ;
# Begin of the program
use warnings;
use strict;
use Getopt::Long qw(:config bundling no_ignore_case);
use Pod::Usage;

use vsetting;

use constant VERSION   => 0;
use constant REVISION  => 3;

# Clear text for transcriber codes in the locator
# Source: text16e6.txt, edited by J.Stolfi at 1998-12-05
my %trans_code = 
    (
     'C' => 'Currier',
     'F' => 'First study group',
     'T' => 'John Tiltman',
     'L' => 'Don Latham',
     'R' => 'Mike Roe',
     'K' => 'Karl Kluge',
     'J' => 'Jim Reed',
     'D' => 'Currier (alternative choice)',
     'G' => 'First study group (alternative choice)',
     'I' => 'Jim Reed (alternative choice)',
     'Q' => 'Karl Kluge (alternative choice)',
     'M' => 'Don Latham (alternative choice)',
     'H' => 'Takeshi Takahashi',
     'N' => 'Gabriel Landini',
     'U' => 'Jorge Stolfi',
     'V' => 'John Grove',
     'P' => 'Father Th. Petersen',
     'X' => 'Denis V. Mardle',
     'Z' => 'Rene Zandbergen'
     );

# Clear text for the illustration type
my %illu_code =
    (
     'T' => 'Text',
     'H' => 'Herbal',
     'A' => 'Astronomical',
     'Z' => 'Zodiac',
     'B' => 'Biological',
     'C' => 'Cosmological',
     'P' => 'Pharmaceutical',
     'S' => 'Stars'
     );

# Initialized globals in the program - change it to fit your defaults
my $eva_filename = $vsetting::evafile;

# The actual locator
my $locator;                        # The locator, raw format
my $loc_folio;                      # The folio number of the locator
my $loc_rv;                         # recto or verso
my $loc_line;                       # The line number
my $loc_trans;                      # The transscriber's code
my $loc_rawtext;                    # The raw text of the locator
my $loc_parsed;                     # Is TRUE after successful parsing

# Page/Unit information
my $pu_raw;                         # Raw information
my $pu_locator;                     # Locator
my $pu_illutype;                    # Illustration type
my $pu_quire;                       # Quire
my $pu_page;                        # Page in quire
my $pu_lang;                        # Currier's language
my $pu_hand;                        # Currier's hand
my $pu_nonvoy;                      # Contains non-voynich text
my $pu_keylike;                     # Contains key-like sequence
my $pu_extra;                       # Contains extraneous writing
my $pu_parsed;                      # Is TRUE after successful parsing

# Options
my ($opt_c, $opt_h, $opt_f, $opt_i, $opt_l,
    $opt_m, $opt_s, $opt_t, $opt_v, $opt_w,
    $opt_C, $opt_F, $opt_I, $opt_L,
    $opt_M, $opt_S, $opt_T);
    
# Option related variables
my $fol_from;                       # Starting folio number
my $fol_to;                         # Ending folio number


# Subroutines
sub show_help
{
    pod2usage(1);
    exit();
}

sub show_transcriber_codes
{
    my ($code, $desc);

    print "Code\tTranscriber\n";
    print "----\t-----------\n";
    foreach $code (sort keys %trans_code)
    {
	$desc = $trans_code{$code};
	print "$code\t$desc\n";
    }
    exit();
}

sub show_illutype_codes
{
    my ($code, $desc);

    print "Code\tDescription\n";
    print "----\t-----------\n";
    foreach $code (sort keys %illu_code)
    {
	$desc = $illu_code{$code};
	print "$code\t$desc\n";
    }
    exit();
}

sub show_version
{
    my $vstr;

    $vstr = sprintf "%d.%02d", VERSION, REVISION;
    print "viat.pl, Version $vstr\n";
    print "Copyright (c) 2004 Michael Winkelmann, michael\@weltretter.de\n";
    exit();
}

sub print_page_header
{
    my ($itype);
    
    $itype = $illu_code{$pu_illutype} if $pu_illutype;
    print "#- Locator           : $pu_locator\n";
    print "#- Illustration type : $itype\n" if $itype;
    print "#- Quire             : $pu_quire\n" if $pu_quire;
    print "#- Page in quire     : $pu_page\n" if $pu_page;
    print "#- Currier language  : $pu_lang\n" if $pu_lang;
    print "#- Currier hand      : $pu_hand\n" if $pu_hand;
    print "#- Page contains non-Voynich text\n" if $pu_nonvoy;
    print "#- Page contains key-like sequence\n" if $pu_keylike;
    print "#- Page contains extraneous writing\n" if $pu_extra;
    print $pu_raw;
}

sub parse_page_header
{
    my ($line) = @_;
    my (%k, $temp);

    $pu_parsed = 0;
    if($line =~ m/\<([^\>]+)\>[^\{]*\{([^\}]+)\}/o)
    {
	$pu_locator = $1;
	$temp = $2;
	$pu_raw = $line;
	while($temp =~ m/\$(.)=(.)/og)
	{
	    $k{$1} = $2;
	}
	if($k{'I'}) { $pu_illutype = $k{'I'}; } else { $pu_illutype = 0; }
	if($k{'Q'}) { $pu_quire = $k{'Q'}; } else { $pu_quire = 0; }
	if($k{'P'}) { $pu_page = $k{'P'}; } else { $pu_page = 0; }
	if($k{'L'}) { $pu_lang = $k{'L'}; } else { $pu_lang = 0; }
	if($k{'H'}) { $pu_hand = $k{'H'}; } else { $pu_hand = 0; }
	$pu_nonvoy = ($k{'N'}) ? 1 : 0;
	$pu_keylike = ($k{'K'}) ? 1 : 0;
	$pu_extra = ($k{'X'}) ? 1 : 0;
	$pu_parsed = 1;
	print_page_header() if($opt_M);
    }
}

sub parse_line
{
    my ($line) = @_;

    $loc_parsed = 0;
    if($line =~ m/^\<([^\>]+)\>\s*(\S.*)$/o)
    {
	$locator = $1;
	$loc_rawtext = $2;
	if($locator =~ m/^f(\d+)(\S+)\.([^\.]+)\.(\d+.?)\;(\S)$/o)
	{
	    $loc_folio = $1;
	    $loc_rv = $2;
	    $loc_line = $4;
	    $loc_trans = $5;
	    $loc_parsed = 1;
	}
    }
}

sub check_for_print
{
    if($opt_t)
    {
	return 0 unless index($opt_t, $loc_trans) >= 0;
    }
    if($opt_f)
    {
	return 0 unless $loc_folio >= $fol_from and $loc_folio <= $fol_to;
    }
    if($opt_i)
    {
	return 0 unless index($opt_i, $pu_illutype) >= 0;
    }
    if($opt_l)
    {
	return 0 unless index($opt_l, $pu_lang) >= 0;
    }
    if($opt_m)
    {
	return 0 unless index($opt_m, $pu_hand) >= 0;
    }
    return 1;
}

sub print_line
{
    my $txt = $loc_rawtext;

    chomp $txt;
    if($opt_w)
    {
	$txt =~ s/.\{\&(\D+?)\}/$1/go;
	$txt =~ s/.\{\&(\d+)\}/pack('c', $1)/goe;
    }
    $txt =~ s/\{[^\}]*\}//go if $opt_C;
    $txt =~ s/\,//go if $opt_S;
    if($opt_s)
    {
	$txt =~ s/\=$/\n/go;
	$txt =~ s/[\.\,\-\=\!\%]/ /go;
    }
    print "<$locator> " unless $opt_L;
    print "$txt\n";
}

# Parse the options
GetOptions('comment_keep|c'               => \$opt_c,
	   'help|h|?'                     => \$opt_h,
	   'folios|folio|f=s'             => \$opt_f,
	   'illustration|i=s'             => \$opt_i,
	   'language|l=s'                 => \$opt_l,
	   'hand|m=s'                     => \$opt_m,
	   'space_convert|s'              => \$opt_s,
	   'transcribers|transcriber|t=s' => \$opt_t,
	   'version|v'                    => \$opt_v,
	   'weirdo_convert|weirdo|w'      => \$opt_w,
	   'comment_suppress|C'           => \$opt_C,
	   'transcription|file|F=s'       => \$opt_F,
	   'show_illustration_codes|I'    => \$opt_I,
	   'locator_suppress|L'           => \$opt_L,
	   'meta_comments|M'              => \$opt_M,
	   'dubious_suppress|S'           => \$opt_S,
	   'show_transcriber_codes|T'     => \$opt_T)
    or die "Illegal option used";

show_help() if $opt_h;
show_transcriber_codes() if $opt_T;
show_illutype_codes() if $opt_I;
show_version() if $opt_v;

# Parse folio range
if($opt_f)
{
    if($opt_f =~ m/(\d+)\:(\d+)/)
    {
	$fol_from = $1;
	$fol_to = $2;
	($fol_from, $fol_to) = ($fol_to, $fol_from) if $fol_from > $fol_to;
    }
    elsif($opt_f =~ m/(\d+)\:/)
    {
	$fol_from = $1;
	$fol_to = 9999;
    }
    elsif($opt_f =~ m/\:(\d+)/)
    {
	$fol_from = 0;
	$fol_to = $1;
    }
    else
    {
	$fol_from = $fol_to = $opt_f;
    }
}

# Initializations
$eva_filename = $opt_F if $opt_F;

# Main program
my $l;
open F, "<$eva_filename" or die "Could not open input file $eva_filename";
while ($l = <F>)
{
    if($l =~ m/^\#/o)
    {
	print $l if($opt_c);
    }
    elsif($l =~ m/\{\s*\$.\=..*\}\s*$/o)
    {
	parse_page_header($l);
    }
    else
    {
	parse_line($l);
	print_line() if check_for_print();
    }
}
close F;
