#!/usr/bin/perl -w
#########################################################################
# vindex.pl
# A simple tool for processing voynich input and creating a full text
# index (a concordance) for the input.
#
# Version 0.02 (beta), 2004-08-16
# Copyright (c) 2004 Michael Winkelmann, michael@weltretter.de
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#########################################################################

=pod

=head1 NAME

vindex.pl - The Voynich interlinear transcription indexer.

=head1 SYNOPSIS

C<vindex.pl [OPTIONS]>

=head1 DESCRIPTION

This program generates an index file from a Voynich interlinear 
transcription archive. This index may be used by other programs or
for analysis purposes as a complete concordance of the Voynich
manuscript.

If you want to do efficient searches with vfind, you should run
this program to create an index.

=head1 OPTIONS

=over 4

=item C<-h> or C<-?> or C<--help>

Show an option overview.

=item C<-f file-spec> or C<--evafile=file-spec>)

Select a transcription file for indexing.

By default, the file 'text16e6.evt' is used.

=item C<-o file-spec> or C<--indexfile>

Select the output file for the generated index.

By default, the file 'vindex.idx' is used.

=item C<-u> or C<--unreadable> or C<--keep_unreadable>

Include all unreadable or partly unreadable words in the
generated index.

An unreadable word is any word which includes a asterisk
character identifing an unreadable glyph.

It does not make much sense to include this words in an
index for searching, but for some analysis purposes this
may be useable, so you can use this option.

=back

=head1 FILES

=over 4

=item text16e6.evt

The interlinear transcription, as provided by J. Stolfi.

=item vindex.idx

The index file. Other tools may expect the existance of this
file.

=back

=head1 INDEX FILE FORMAT

The format of the index file is rather straightforward and easy
to use with all the popular un*x tools (awk, perl). It serves as
a complete concordance of the manuscript.

There is a line for every word, the items are separated by
spaces.

The line starts with the word, followed by all locators of the
transcription where this word can be found.

=head1 AUTHOR

Michael Winkelmann, michael@weltretter.de

=cut

use warnings;
use strict;
use Getopt::Long qw(:config bundling no_ignore_case);
use Pod::Usage;
use vsetting;

my ($opt_keepunreadable, $opt_help);

my %index;
my $evafile = $vsetting::evafile;
my $idxfile = $vsetting::indexfile;
my ($line, $locator, $text, $word);

GetOptions(
    'help|h|?'                            => \$opt_help,
    'evafile|f=s'                         => \$evafile,
    'indexfile|o=s'                       => \$idxfile,
    'keep_unreadable|unreadable|u'        => \$opt_keepunreadable)
    or die 'Illegal option used';

# Display help text
if($opt_help)
{
    pod2usage(1);
    exit();
}

# Build the index
open IF, "<$evafile" or die "Could not open EVA file $evafile for reading";
while($line = <IF>)
{
    # Ignore comments and parseable information
    next if $line =~ m/^\#/o or $line =~ m/\{\s*\$.\=/o;

    # Split the line in locator and text
    next unless $line =~ m/^\<([^\>]+)\>\s+(.+)$/o;
    $locator = $1;
    $text = $2;

    # Remove line end markers, special spaces and inline comments from the line
    $text =~ s/[-=!%]//go;
    $text =~ s/[.,]/ /go;
    $text =~ s/\{[^\}]*}//go;

    # Split the text in words and process these words
    foreach $word (split / /, $text)
    {
	# Ignore unreadable words
	unless($opt_keepunreadable)
	{
	    next if $word =~ m/\*/o;
	}

	# Add the locator to the index
	push @{$index{$word}}, $locator;
    }
}
close IF or die "Could not close input file";

# Print the index
open OF, ">$idxfile" or die "Could not open index file $idxfile for writing";
foreach $word (sort keys %index)
{
    print OF "$word ", join(' ', @{$index{$word}}), "\n"; 
}
close OF or die "Could not close index file";

